/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2020, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *

*** Generated by yangdump-sdk

    User SIL module
    module ietf-interfaces
    revision 2014-05-08
    namespace urn:ietf:params:xml:ns:yang:ietf-interfaces
    organization IETF NETMOD (NETCONF Data Modeling Language) Working Group
    Created: 2020-03-05T01:12:07Z
    CLI parameters:
        format uc
        indent 4
        log-level debug3
        module ietf-interfaces
        output u_ietf-interfaces.c
        sil-edit2
        sil-get2
        unified true

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_cfg.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "getcb.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "obj.h"
#include "rpc.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_val.h"
#include "u_ietf-interfaces.h"
#include "y_ietf-interfaces.h"


/* put your static variables here */


/********************************************************************
* FUNCTION u_ietf_interfaces_interfaces_interface_edit
*
* Edit database object callback
* Path: list /interfaces/interface
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*     k_ parameters are ancestor list key values.
*
* RETURNS:
*     error status
********************************************************************/
status_t u_ietf_interfaces_interfaces_interface_edit (
    ses_cb_t *scb,
    rpc_msg_t *msg,
    agt_cbtyp_t cbtyp,
    op_editop_t editop,
    val_value_t *newval,
    val_value_t *curval,
    const xmlChar *k_interfaces_interface_name)
{
    /* uncomment if these parameters are used */
    (void)scb;
    (void)k_interfaces_interface_name;

    status_t res = NO_ERR;

    if (LOGDEBUG) {
        log_debug("\nEnter u_ietf_interfaces_interfaces_interface_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    if (editop == OP_EDITOP_MERGE) {
        /* the edit is not really on this node; need to get
         * each child_undo record to get the real edited nodes */
        agt_cfg_transaction_t *txcb = RPC_MSG_TXCB(msg);
        agt_cfg_undo_rec_t *child_edit =
            agt_cfg_first_child_edit(txcb, newval, curval);

        while (child_edit) {
            op_editop_t child_editop = OP_EDITOP_NONE;
            val_value_t *child_newval = NULL;
            val_value_t *child_curval = NULL;

            agt_cfg_child_edit_fields(child_edit,
                &child_editop,
                &child_newval,
                &child_curval);

            if (LOGDEBUG2) {
                log_debug2("\nchild edit %s", op_editop_name(child_editop));
                if (child_newval) {
                    log_debug2_append("\nnew");
                    val_dump_value(child_newval, 0, DBG2);
                }
                if (child_curval) {
                    log_debug2_append("\ncur");
                    val_dump_value(child_curval, 0, DBG2);
                }
            }

            obj_template_t *chobj = (child_newval) ?
                VAL_OBJ(child_newval) : VAL_OBJ(child_curval);
            const xmlChar *name = obj_get_name(chobj);

            /**** process child edits here ****/

            if (!xml_strcmp(name, y_ietf_interfaces_N_description)) {
                /* leaf description (string) */

            } else if (!xml_strcmp(name, y_ietf_interfaces_N_type)) {
                /* leaf type (identityref) */

            } else if (!xml_strcmp(name, y_ietf_interfaces_N_enabled)) {
                /* leaf enabled (boolean) */

            } else if (!xml_strcmp(name, y_ietf_interfaces_N_link_up_down_trap_enable)) {
                /* leaf link-up-down-trap-enable (enumeration) */

            }

            child_edit = agt_cfg_next_child_edit(child_edit);
        }
    }

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }
    return res;

} /* u_ietf_interfaces_interfaces_interface_edit */


/********************************************************************
* FUNCTION u_ietf_interfaces_interfaces_edit
*
* Edit database object callback
* Path: container /interfaces
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
status_t u_ietf_interfaces_interfaces_edit (
    ses_cb_t *scb,
    rpc_msg_t *msg,
    agt_cbtyp_t cbtyp,
    op_editop_t editop,
    val_value_t *newval,
    val_value_t *curval)
{

    /* uncomment if these parameters are used */
    (void)scb;
    (void)msg;
    (void)newval;
    (void)curval;

    status_t res = NO_ERR;

    if (LOGDEBUG) {
        log_debug("\nEnter u_ietf_interfaces_interfaces_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }
    return res;

} /* u_ietf_interfaces_interfaces_edit */


/********************************************************************
* FUNCTION u_ietf_interfaces_interfaces_state_interface_statistics_get
*
* Get database object callback for container statistics
* Path: container /interfaces-state/interface/statistics
* Fill in 'get2cb' response fields
*
* INPUTS:
*     see ncx/getcb.h for details (getcb_fn2_t)
*
* RETURNS:
*     error status
********************************************************************/
status_t u_ietf_interfaces_interfaces_state_interface_statistics_get (
    getcb_get2_t *get2cb,
    const xmlChar *k_interfaces_state_interface_name)
{
    /* uncomment if these parameters are used */
    (void)k_interfaces_state_interface_name;

    /* check the callback mode type */
    getcb_mode_t cbmode = GETCB_GET2_CBMODE(get2cb);
    switch (cbmode) {
    case GETCB_GET_VALUE:
        break;
    case GETCB_GETNEXT_VALUE:
        return ERR_NCX_NO_INSTANCE;
    default:
        return SET_ERROR(ERR_INTERNAL_VAL);
    }

    /* an NP container always exists so no test for node_exists
     * by the SIL or SIL-SA callback is needed */
    obj_template_t *obj = GETCB_GET2_OBJ(get2cb);
    status_t res = NO_ERR;


    /* go through all the requested terminal child objects */
    obj_template_t *childobj =
        getcb_first_requested_child(get2cb, obj);
    for (; childobj; childobj =
        getcb_next_requested_child(get2cb, childobj)) {

        const xmlChar *name = obj_get_name(childobj);
        val_value_t *retval = NULL;

        /* Retrieve the value of this terminal node and
         * add with getcb_add_return_val */

        if (!xml_strcmp(name, y_ietf_interfaces_N_discontinuity_time)) {
            /* leaf discontinuity-time (string) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_in_octets)) {
            /* leaf in-octets (uint64) */

            retval =
                val_make_simval_obj(childobj,
                                    (const xmlChar *)"1000",
                                    &res);
            if (retval) {
                getcb_add_return_val(get2cb, retval);
            }

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_in_unicast_pkts)) {
            /* leaf in-unicast-pkts (uint64) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_in_broadcast_pkts)) {
            /* leaf in-broadcast-pkts (uint64) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_in_multicast_pkts)) {
            /* leaf in-multicast-pkts (uint64) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_in_discards)) {
            /* leaf in-discards (uint32) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_in_errors)) {
            /* leaf in-errors (uint32) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_in_unknown_protos)) {
            /* leaf in-unknown-protos (uint32) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_out_octets)) {
            /* leaf out-octets (uint64) */

            retval =
                val_make_simval_obj(childobj,
                                    (const xmlChar *)"0",
                                    &res);
            if (retval) {
                getcb_add_return_val(get2cb, retval);
            }

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_out_unicast_pkts)) {
            /* leaf out-unicast-pkts (uint64) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_out_broadcast_pkts)) {
            /* leaf out-broadcast-pkts (uint64) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_out_multicast_pkts)) {
            /* leaf out-multicast-pkts (uint64) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_out_discards)) {
            /* leaf out-discards (uint32) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_out_errors)) {
            /* leaf out-errors (uint32) */

        }
    }

    return res;

} /* u_ietf_interfaces_interfaces_state_interface_statistics_get */


/********************************************************************
* FUNCTION u_ietf_interfaces_interfaces_state_interface_get
*
* Get database object callback for list interface
* Path: list /interfaces-state/interface
* Fill in 'get2cb' response fields
*
* INPUTS:
*     see ncx/getcb.h for details (getcb_fn2_t)
*
* RETURNS:
*     error status
********************************************************************/
status_t u_ietf_interfaces_interfaces_state_interface_get (
    getcb_get2_t *get2cb,
    const xmlChar *k_interfaces_state_interface_name,
    boolean name_fixed,
    boolean name_present)
{

    /* uncomment if these parameters are used */
    (void)k_interfaces_state_interface_name;
    (void)name_present;

    boolean getnext = FALSE;

    /* check the callback mode type */
    getcb_mode_t cbmode = GETCB_GET2_CBMODE(get2cb);
    switch (cbmode) {
    case GETCB_GET_VALUE:
        break;
    case GETCB_GETNEXT_VALUE:
        getnext = TRUE;
        break;
    default:
        return SET_ERROR(ERR_INTERNAL_VAL);
    }

    obj_template_t *obj = GETCB_GET2_OBJ(get2cb);
    status_t res = NO_ERR;


    /* uncomment if this parameter is used */
    (void)getnext;

    /* For GET, find the entry that matches the key values
     * For GETNEXT, find the entry that matches the next key value
     * If the 'present' flag is false then return first key instance
     * If the 'fixed' flag is true then no GETNEXT advance for the key
     * Create a new return key val_value_t, then getcb_add_return_key */

    /***** ADD RETURN KEYS AND REMOVE THIS COMMENT ****/


    /* Pretend there is only one interface entry with name vlan1
     * This is just a simpified example.
     */
    obj_template_t *name_obj =
        obj_find_child(obj,
                       y_ietf_interfaces_M_ietf_interfaces,
                       (const xmlChar *)"name");

    val_value_t *retval =
        val_make_simval_obj(name_obj,
                            (const xmlChar *)"vlan1",
                            &res);
    if (retval) {
        if (name_fixed) {
            VAL_SET_FIXED_VALUE(retval);
        }
        getcb_add_return_key(get2cb, retval);
    }

    if (GETCB_GET2_FIRST_RETURN_KEY(get2cb) == NULL) {
        return ERR_NCX_NO_INSTANCE;
    }

    /* For GETNEXT, set the more_data flag to TRUE
     * This will be just a single entry in the list so set
     * more_data to FALSE.
     */
    GETCB_GET2_MORE_DATA(get2cb) = FALSE;


    /* go through all the requested terminal child objects */
    obj_template_t *childobj =
        getcb_first_requested_child(get2cb, obj);
    for (; childobj; childobj =
        getcb_next_requested_child(get2cb, childobj)) {

        const xmlChar *name = obj_get_name(childobj);

        log_debug("\n\n ****** HERE IS SOME TEXT THAT STANDS OUT! ******\n");

        /* Retrieve the value of this terminal node and
         * add with getcb_add_return_val */

        if (!xml_strcmp(name, y_ietf_interfaces_N_type)) {
            /* leaf type (identityref) */

            retval =
                val_make_simval_obj(childobj,
                                    (const xmlChar *)"ianaift:l2vlan",
                                    &res);
            if (retval) {
                getcb_add_return_val(get2cb, retval);
            }

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_admin_status)) {
            /* leaf admin-status (enumeration) */

            retval =
                val_make_simval_obj(childobj,
                                    (const xmlChar *)"up",
                                    &res);
            if (retval) {
                getcb_add_return_val(get2cb, retval);
            }

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_oper_status)) {
            /* leaf oper-status (enumeration) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_last_change)) {
            /* leaf last-change (string) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_if_index)) {
            /* leaf if-index (int32) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_phys_address)) {
            /* leaf phys-address (string) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_higher_layer_if)) {
            /* leaf-list higher-layer-if (leafref) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_lower_layer_if)) {
            /* leaf-list lower-layer-if (leafref) */

        } else if (!xml_strcmp(name, y_ietf_interfaces_N_speed)) {
            /* leaf speed (uint64) */

            retval =
                agt_make_uint64_leaf(obj,
                                     name,
                                     (uint64)500,
                                     &res);
            if (retval) {
                getcb_add_return_val(get2cb, retval);
            }
        }
    }

    return res;

} /* u_ietf_interfaces_interfaces_state_interface_get */


/********************************************************************
* FUNCTION u_ietf_interfaces_interfaces_state_get
*
* Get database object callback for container interfaces-state
* Path: container /interfaces-state
* Fill in 'get2cb' response fields
*
* INPUTS:
*     see ncx/getcb.h for details (getcb_fn2_t)
*
* RETURNS:
*     error status
********************************************************************/
status_t
    u_ietf_interfaces_interfaces_state_get (getcb_get2_t *get2cb)
{
    /* check the callback mode type */
    getcb_mode_t cbmode = GETCB_GET2_CBMODE(get2cb);
    switch (cbmode) {
    case GETCB_GET_VALUE:
        break;
    case GETCB_GETNEXT_VALUE:
        return ERR_NCX_NO_INSTANCE;
    default:
        return SET_ERROR(ERR_INTERNAL_VAL);
    }

    /* an NP container always exists so no test for node_exists
     * by the SIL or SIL-SA callback is needed */
    obj_template_t *obj = GETCB_GET2_OBJ(get2cb);
    status_t res = NO_ERR;

    /* uncomment if this parameter is used */
    (void)obj;

    /* no terminal child nodes to check */

    return res;

} /* u_ietf_interfaces_interfaces_state_get */


/********************************************************************
* FUNCTION u_ietf_interfaces_init
*
* initialize the ietf-interfaces server instrumentation library
*
* INPUTS:
*    modname == requested module name
*    revision == requested version (NULL for any)
*
* RETURNS:
*     error status
********************************************************************/
status_t
    u_ietf_interfaces_init (const xmlChar *modname,
                            const xmlChar *revision)
{
    status_t res = NO_ERR;
    ncx_module_t *ietf_interfaces_mod = NULL;

    ietf_interfaces_mod = ncx_find_module(modname, revision);
    if (ietf_interfaces_mod == NULL) {
        return ERR_NCX_OPERATION_FAILED;
    }

    /* put your module initialization code here */

    return res;

} /* u_ietf_interfaces_init */

/********************************************************************
* FUNCTION u_ietf_interfaces_init2
*
* SIL init phase 2: non-config data structures
* Called after running config is loaded
*
* RETURNS:
*     error status
********************************************************************/
status_t u_ietf_interfaces_init2 (void)
{
    status_t res = NO_ERR;

    /* put your init2 code here */

    return res;

} /* u_ietf_interfaces_init2 */

/********************************************************************
* FUNCTION u_ietf_interfaces_cleanup
*    cleanup the server instrumentation library
*
********************************************************************/
void u_ietf_interfaces_cleanup (void)
{

    /* put your cleanup code here */

} /* u_ietf_interfaces_cleanup */

/* END SIL u_ietf_interfaces.c */
