/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2020, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *

*** Generated by yangdump-sdk

    YumaPro SIL module
    module ietf-interfaces
    revision 2014-05-08
    namespace urn:ietf:params:xml:ns:yang:ietf-interfaces
    organization IETF NETMOD (NETCONF Data Modeling Language) Working Group
    Created: 2020-03-05T01:12:07Z
    CLI parameters:
        format yc
        indent 4
        log-level debug3
        module ietf-interfaces
        output y_ietf-interfaces.c
        sil-edit2
        sil-get2
        unified true

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_cfg.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "getcb.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "obj.h"
#include "rpc.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_val.h"
#include "u_ietf-interfaces.h"
#include "y_ietf-interfaces.h"

/* module static variables */
static ncx_module_t *ietf_interfaces_mod;
static obj_template_t *interfaces_obj;
static obj_template_t *interfaces_state_obj;
static val_value_t *interfaces_val;

/********************************************************************
* FUNCTION y_ietf_interfaces_init_static_vars
*
* initialize module static variables
*
********************************************************************/
static void y_ietf_interfaces_init_static_vars (void)
{
    ietf_interfaces_mod = NULL;
    interfaces_obj = NULL;
    interfaces_state_obj = NULL;
    interfaces_val = NULL;

} /* y_ietf_interfaces_init_static_vars */


/********************************************************************
* FUNCTION ietf_interfaces_interfaces_interface_edit
*
* Edit database object callback
* Path: list /interfaces/interface
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    ietf_interfaces_interfaces_interface_edit (ses_cb_t *scb,
                                               rpc_msg_t *msg,
                                               agt_cbtyp_t cbtyp,
                                               op_editop_t editop,
                                               val_value_t *newval,
                                               val_value_t *curval)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;
    uint32 cur_errs = agt_get_error_count(msg);
    val_value_t *lastkey = NULL;

    /* local key ietf-interfaces:name */
    const xmlChar *k_interfaces_interface_name =
        VAL_STRING(agt_get_key_value(errorval, &lastkey));

    if (LOGDEBUG) {
        log_debug("\nEnter ietf_interfaces_interfaces_interface_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    res =
        u_ietf_interfaces_interfaces_interface_edit(scb,
                                                    msg,
                                                    cbtyp,
                                                    editop,
                                                    newval,
                                                    curval,
                                                    k_interfaces_interface_name);

    if ((res != NO_ERR) && (cur_errs == agt_get_error_count(msg))) {
        agt_record_error(scb,
                         &msg->mhdr,
                         NCX_LAYER_CONTENT,
                         res,
                         NULL,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval);
    }
    return res;

} /* ietf_interfaces_interfaces_interface_edit */


/********************************************************************
* FUNCTION ietf_interfaces_interfaces_edit
*
* Edit database object callback
* Path: container /interfaces
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    ietf_interfaces_interfaces_edit (ses_cb_t *scb,
                                     rpc_msg_t *msg,
                                     agt_cbtyp_t cbtyp,
                                     op_editop_t editop,
                                     val_value_t *newval,
                                     val_value_t *curval)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;
    uint32 cur_errs = agt_get_error_count(msg);

    if (LOGDEBUG) {
        log_debug("\nEnter ietf_interfaces_interfaces_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    res = u_ietf_interfaces_interfaces_edit(scb,
                                            msg,
                                            cbtyp,
                                            editop,
                                            newval,
                                            curval);

    if (res == NO_ERR && cbtyp == AGT_CB_COMMIT) {
        res = agt_check_cache(&interfaces_val,
                              newval,
                              curval,
                              editop);
    }


    if ((res != NO_ERR) && (cur_errs == agt_get_error_count(msg))) {
        agt_record_error(scb,
                         &msg->mhdr,
                         NCX_LAYER_CONTENT,
                         res,
                         NULL,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval,
                         (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
                         errorval);
    }
    return res;

} /* ietf_interfaces_interfaces_edit */


/********************************************************************
* FUNCTION ietf_interfaces_interfaces_state_interface_statistics_get
*
* Get database object callback for container statistics
* Path: container /interfaces-state/interface/statistics
* Fill in 'get2cb' response fields
*
* INPUTS:
*     see ncx/getcb.h for details (getcb_fn2_t)
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    ietf_interfaces_interfaces_state_interface_statistics_get (
        ses_cb_t *scb,
        xml_msg_hdr_t *msg,
        getcb_get2_t *get2cb)
{

    (void)scb;
    (void)msg;
    val_value_t *keyval = NULL;

    /* ancestor key ietf-interfaces:name */
    const xmlChar *k_interfaces_state_interface_name = 0;
    keyval = GETCB_GET2_FIRST_KEY(get2cb);
    if (keyval) {
        k_interfaces_state_interface_name = VAL_STRING(keyval);
    }

    return u_ietf_interfaces_interfaces_state_interface_statistics_get(
        get2cb,
        k_interfaces_state_interface_name);

} /* ietf_interfaces_interfaces_state_interface_statistics_get */

/********************************************************************
* FUNCTION ietf_interfaces_interfaces_state_interface_get
*
* Get database object callback for list interface
* Path: list /interfaces-state/interface
* Fill in 'get2cb' response fields
*
* INPUTS:
*     see ncx/getcb.h for details (getcb_fn2_t)
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    ietf_interfaces_interfaces_state_interface_get (
        ses_cb_t *scb,
        xml_msg_hdr_t *msg,
        getcb_get2_t *get2cb)
{

    (void)scb;
    (void)msg;
    val_value_t *keyval = NULL;

    /* local key ietf-interfaces:name */
    const xmlChar *k_interfaces_state_interface_name = 0;
    boolean name_fixed = FALSE;
    boolean name_present = FALSE;
    keyval =
        getcb_find_key_lvl(get2cb,
                           y_ietf_interfaces_M_ietf_interfaces,
                           y_ietf_interfaces_N_name,
                           (uint32)3);
    if (keyval) {
        k_interfaces_state_interface_name = VAL_STRING(keyval);
        name_fixed = VAL_IS_FIXED_VALUE(keyval);
        name_present = TRUE;
    }

    return u_ietf_interfaces_interfaces_state_interface_get(
        get2cb,
        k_interfaces_state_interface_name,
        name_fixed,
        name_present);

} /* ietf_interfaces_interfaces_state_interface_get */


/********************************************************************
* FUNCTION ietf_interfaces_interfaces_state_get
*
* Get database object callback for container interfaces-state
* Path: container /interfaces-state
* Fill in 'get2cb' response fields
*
* INPUTS:
*     see ncx/getcb.h for details (getcb_fn2_t)
*
* RETURNS:
*     error status
********************************************************************/
static status_t ietf_interfaces_interfaces_state_get (
    ses_cb_t *scb,
    xml_msg_hdr_t *msg,
    getcb_get2_t *get2cb)
{

    (void)scb;
    (void)msg;
    return u_ietf_interfaces_interfaces_state_get(get2cb);

} /* ietf_interfaces_interfaces_state_get */


/********************************************************************
* FUNCTION y_ietf_interfaces_init
*
* initialize the ietf-interfaces server instrumentation library
*
* INPUTS:
*    modname == requested module name
*    revision == requested version (NULL for any)
*
* RETURNS:
*     error status
********************************************************************/
status_t
    y_ietf_interfaces_init (const xmlChar *modname,
                            const xmlChar *revision)
{
    status_t res = NO_ERR;

    y_ietf_interfaces_init_static_vars();

    /* change if custom handling done */
    if (xml_strcmp(modname, y_ietf_interfaces_M_ietf_interfaces)) {
        return ERR_NCX_UNKNOWN_MODULE;
    }

    if (revision && xml_strcmp(revision, y_ietf_interfaces_R_ietf_interfaces)) {
        return ERR_NCX_WRONG_VERSION;
    }

    res =
        ncx_set_feature_enable(y_ietf_interfaces_M_ietf_interfaces,
                              (const xmlChar *)"arbitrary-names",
#ifdef u_ietf_interfaces_F_arbitrary_names
                               true);
#else
                               false);
#endif
    if (res != NO_ERR) {
        return res;
    }

    res =
        ncx_set_feature_enable(y_ietf_interfaces_M_ietf_interfaces,
                               (const xmlChar *)"pre-provisioning",
#ifdef u_ietf_interfaces_F_pre_provisioning
                               true);
#else
                               false);
#endif
    if (res != NO_ERR) {
        return res;
    }

    res =
        ncx_set_feature_enable(y_ietf_interfaces_M_ietf_interfaces,
                               (const xmlChar *)"if-mib",
#ifdef u_ietf_interfaces_F_if_mib
                               true);
#else
                               false);
#endif
    if (res != NO_ERR) {
        return res;
    }

    res =
        ncxmod_load_module(y_ietf_interfaces_M_ietf_interfaces,
                           y_ietf_interfaces_R_ietf_interfaces,
                           agt_get_savedevQ(),
                           &ietf_interfaces_mod);
    if (res != NO_ERR) {
        return res;
    }

    interfaces_obj =
        ncx_find_object(ietf_interfaces_mod,
                        y_ietf_interfaces_N_interfaces);
    if (interfaces_obj == NULL) {
        return ERR_NCX_DEF_NOT_FOUND;
    }

    interfaces_state_obj =
        ncx_find_object(ietf_interfaces_mod,
                        y_ietf_interfaces_N_interfaces_state);
    if (interfaces_state_obj == NULL) {
        return ERR_NCX_DEF_NOT_FOUND;
    }

    res =
        agt_cb_register_edit2_callback(y_ietf_interfaces_M_ietf_interfaces,
                                       (const xmlChar *)"/if:interfaces",
                                       y_ietf_interfaces_R_ietf_interfaces,
                                       ietf_interfaces_interfaces_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_edit2_callback(y_ietf_interfaces_M_ietf_interfaces,
                                       (const xmlChar *)"/if:interfaces/if:interface",
                                       y_ietf_interfaces_R_ietf_interfaces,
                                       ietf_interfaces_interfaces_interface_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_get_callback(y_ietf_interfaces_M_ietf_interfaces,
                                     (const xmlChar *)"/if:interfaces-state",
                                     y_ietf_interfaces_R_ietf_interfaces,
                                     ietf_interfaces_interfaces_state_get);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_get_callback(y_ietf_interfaces_M_ietf_interfaces,
                                     (const xmlChar *)"/if:interfaces-state/if:interface",
                                     y_ietf_interfaces_R_ietf_interfaces,
                                     ietf_interfaces_interfaces_state_interface_get);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_get_callback(y_ietf_interfaces_M_ietf_interfaces,
                                     (const xmlChar *)"/if:interfaces-state/if:interface/if:statistics",
                                     y_ietf_interfaces_R_ietf_interfaces,
                                     ietf_interfaces_interfaces_state_interface_statistics_get);
    if (res != NO_ERR) {
        return res;
    }

    res = u_ietf_interfaces_init(modname, revision);
    return res;

} /* y_ietf_interfaces_init */

/********************************************************************
* FUNCTION y_ietf_interfaces_init2
*
* SIL init phase 2: non-config data structures
* Called after running config is loaded
*
* RETURNS:
*     error status
********************************************************************/
status_t y_ietf_interfaces_init2 (void)
{
    status_t res = NO_ERR;

    interfaces_val =
        agt_init_cache(y_ietf_interfaces_M_ietf_interfaces,
                       y_ietf_interfaces_N_interfaces,
                       &res);
    if (res != NO_ERR) {
        return res;
    }

    res = u_ietf_interfaces_init2();

    return res;

} /* y_ietf_interfaces_init2 */

/********************************************************************
* FUNCTION y_ietf_interfaces_cleanup
*    cleanup the server instrumentation library
*
********************************************************************/
void y_ietf_interfaces_cleanup (void)
{

    agt_cb_unregister_callbacks(y_ietf_interfaces_M_ietf_interfaces,
                                (const xmlChar *)"/if:interfaces");

    agt_cb_unregister_callbacks(y_ietf_interfaces_M_ietf_interfaces,
                                (const xmlChar *)"/if:interfaces/if:interface");

    agt_cb_unregister_callbacks(y_ietf_interfaces_M_ietf_interfaces,
                                (const xmlChar *)"/if:interfaces-state");

    agt_cb_unregister_callbacks(y_ietf_interfaces_M_ietf_interfaces,
                                (const xmlChar *)"/if:interfaces-state/if:interface");

    agt_cb_unregister_callbacks(y_ietf_interfaces_M_ietf_interfaces,
                                (const xmlChar *)"/if:interfaces-state/if:interface/if:statistics");

    u_ietf_interfaces_cleanup();

} /* y_ietf_interfaces_cleanup */

/* END SIL y_ietf_interfaces.c */
